:- use_module(library(clpfd)).
:- encoding(utf8).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Practica programacion declarativa avanzada. Calculador de dietas

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Realizada por:
	%Beatriz Gomez Carrero
	%Concepcion Nuñez Montes de Oca
	
%Ficheros adicionales:
	%menuSemanal: En el se escribe el resultado de una dieta para 7 dias
	%dieta: En el se escribe el resultado para una dieta de 1 dia
	%alergiasYpreferencias: 
		%En el se pueden especificar alimentos no permitidos mediante alergia(nombreAlimento)
		% o alimentos preferentes mediante preferente(nombreAlimento)
	%ingredientes: Se utiliza para las alergias y preferencias almacena los ingredientes de cada plato
	%aserts: En el se encuentran la informacion nutricional de todos los alimentos.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LLamadas a metodos principales %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MENU 7 DIAS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% INTRODUCIENDO SOLO NUMERO DE KCAL %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

menu_semanal(Kcal):- Kcal<0, print_message(error, 'El numero de kcal debe ser un valor positivo'),!.
menu_semanal(Kcal):- calcula_menu_semanal(Kcal, 0.325, 0.505, 0.225),!.
%Ej:	menu_semanal(1400).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% INTRODUCIENDO KCAL Y PORCENTAJE DE PROTEINAS, GRASAS E HIDRATOS (Porcentajes en tanto por uno) %%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

menu_semanal_con_porcentajes(Kcal, _, _, _):- Kcal<0, print_message(error, 'El numero de kcal debe ser un valor positivo\n'),!.
menu_semanal_con_porcentajes(_, PH, PP, PG):- (PH+PP+PG)<1, print_message(error, 'La suma de porcentajes debe ser igual a 1 \n'),!.
menu_semanal_con_porcentajes(Kcal, PH, PP, PG):- calcula_menu_semanal(Kcal, PP, PH, PG),!.
%Ej:	menu_semanal_con_porcentajes(1400, 0.6, 0.2, 0.2).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% CALCULANDO METABOLISMO BASAL: INTRODUCIENDO SEXO, PESO(KG) ALTURA(CM) EDAD Y ACTIVIDAD FISICA %%%%%
%%%%%  			Sexo: [h, m], Actividad fisica : [sedentaria, ligera, activa, muy activa]			%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


menu_semanal_con_metabolismo_basal(Sexo, _, _, _, _):-  not(member(Sexo, [h,m])), print_message(error, 'El sexo debe ser uno de los siguientes valores [h, m]\n'),!.
menu_semanal_con_metabolismo_basal(_, Peso, _, _, _):- Peso<0, print_message(error, 'El peso debe ser un valor positivo (en kg)\n'),!.
menu_semanal_con_metabolismo_basal(_, _, Altura, _, _):- Altura=<50, print_message(error, 'La altura debe ser un valor mayor que 50 (en cm)\n'),!.
menu_semanal_con_metabolismo_basal(_, _, _, Edad, _):- Edad=<5, print_message(error, 'La edad debe ser un valor mayor que 5 (en años)\n'),!.
menu_semanal_con_metabolismo_basal(_, _, _, _, Actividad):- not(member(Actividad,[sedentaria, ligera, media, activa, muyActiva])), print_message(error, 'La actividad fisica debe ser uno de los siguientes valores [sedentaria, ligera, media, activa, muyActiva])\n'),!.
menu_semanal_con_metabolismo_basal(Sexo, Peso, Altura, Edad, Actividad):- 
																		calcula_Kcal(Sexo, Peso, Altura, Edad, Actividad, Kcal),
																		calcula_menu_semanal(Kcal, 0.325, 0.505, 0.225),!.
%Ej: 	menu_semanal_con_metabolismo_basal(m, 57,164, 23, activa).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MENU 1 DIA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% INTRODUCIENDO SOLO NUMERO DE KCAL %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

menu_diario(Kcal):- Kcal<0, print_message(error, 'El numero de kcal debe ser un valor positivo'),!.
menu_diario(Kcal):- calcula_menu(Kcal, 0.325, 0.505, 0.225),!.
%Ej:	menu_diario(1700).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% INTRODUCIENDO KCAL Y PORCENTAJE DE PROTEINAS, GRASAS E HIDRATOS (Porcentajes en tanto por uno) %%%%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

menu_diario_con_porcentajes(Kcal, _, _, _):- Kcal<0, print_message(error, 'El numero de kcal debe ser un valor positivo\n'),!.
menu_diario_con_porcentajes(_, PH, PP, PG):- (PH+PP+PG)<1, print_message(error, 'La suma de porcentajes debe ser igual a 1 \n'),!.
menu_diario_con_porcentajes(Kcal, PH, PP, PG):- calcula_menu(Kcal, PP, PH, PG),!.
%Ej:	menu_diario_con_porcentajes(1700, 0.6, 0.3, 0.1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% CALCULANDO METABOLISMO BASAL: INTRODUCIENDO SEXO, PESO(KG) ALTURA(CM) EDAD Y ACTIVIDAD FISICA %%%%%
%%%%%  			Sexo: [h, m], Actividad fisica : [sedentaria, ligera, activa, muy activa]			%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

menu_diario_con_metabolismo_basal(Sexo, _, _, _, _):-  not(member(Sexo, [h,m])), print_message(error, 'El sexo debe ser uno de los siguientes valores [h, m]\n'),!.
menu_diario_con_metabolismo_basal(_, Peso, _, _, _):- Peso<0, print_message(error, 'El peso debe ser un valor positivo (en kg)\n'),!.
menu_diario_con_metabolismo_basal(_, _, Altura, _, _):- Altura=<50, print_message(error, 'La altura debe ser un valor mayor que 50 (en cm)\n'),!.
menu_diario_con_metabolismo_basal(_, _, _, Edad, _):- Edad=<5, print_message(error, 'La edad debe ser un valor mayor que 5 (en años)\n'),!.
menu_diario_con_metabolismo_basal(_, _, _, _, Actividad):- not(member(Actividad,[sedentaria, ligera, media, activa, muyActiva])), print_message(error, 'La actividad fisica debe ser uno de los siguientes valores [sedentaria, ligera, media, activa, muyActiva])\n'),!.
menu_diario_con_metabolismo_basal(Sexo, Peso, Altura, Edad, Actividad):-
																		calcula_Kcal(Sexo, Peso, Altura, Edad, Actividad, Kcal),
																		calcula_menu(Kcal, 0.325, 0.505, 0.225),!.
%Ej:	menu_diario_con_metabolismo_basal(h, 70,183, 25, ligera).
																		
																																		
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:-consult(aserts), consult(permitidos), consult(ingredientes), consult(alergiasYpreferencias).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Metodo principal que calcula los platos de cada comida
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%calcula_comidas(+Kcal,+PorcProteinas,+PorcHidratos,+PorcGrasas,-Desayuno,-MediaManana,-Comida,-Merienda,-Cena,-KCalTotal,-ProteinasTotal,-HidratosTotal,-GrasasTotal)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

calcula_comidas(Kcal,  PorcProteinas, PorcHidratos, PorcGrasas, Desayuno, MediaManana, Comida, Merienda, Cena, KCalTotal, ProteinasTotal, HidratosTotal, GrasasTotal):-
		%Calculamos porcentajes caloricos
		A is truncate(Kcal*0.10), B is truncate(Kcal*0.20),
		C is truncate(Kcal*0.25), D is truncate(Kcal*0.30),
		E is truncate(Kcal*0.35), F is truncate(Kcal*0.40),
		%Ponemos las restricciones caloricas
		MinKCalDesayuno is B, MaxKCalDesayuno is D,
		MinKCalMediaManana is 100, MaxKCalMediaManana is 200,
		MinKCalComida is E, MaxKCalComida is F,
		MinKCalMerienda is A , MaxKCalMerienda is B,
		MinKCalCena is C, MaxKCalCena is D,
		
		%Calculamos los gramos que hay que comer de proteinas, grasas e hidratos de carbono
		calcula_cantidades(Kcal, PorcProteinas, PorcHidratos, PorcGrasas, GrasasMin, ProteinasMin, HidratosMin, GrasasMax, ProteinasMax, HidratosMax),

		%Copiamos una lista de todos los alimentos disponibles en la base de conocimiento para cada comida
		buscaComida(0, Kcal, desayuno, ListaDesayuno),
		buscaComida(0, Kcal, mediaManana,  ListaMediaManana),
		buscaComida(0, Kcal, comida, ListaComida),
		buscaComida(0, Kcal, merienda, ListaMerienda),
		buscaComida(0, Kcal, cena, ListaCena),

		%Permutamos las listas para que el resultado no siempre sea el mismo
		random_permutation(ListaDesayuno, ListaDesayunoP),
		random_permutation(ListaMediaManana, ListaMediaMananaP),
		random_permutation(ListaComida, ListaComidaP),
		random_permutation(ListaMerienda, ListaMeriendaP),
		random_permutation(ListaCena, ListaCenaP),
		
		%Hacemos posible que se muestren varias comidas, es decir, cada comida de el dia puede contener mas de un plato
		listaComida(ListaDesayunoP, desayuno, MaxKCalDesayuno-MinKCalDesayuno, ((MaxKCalDesayuno+MinKCalDesayuno)/2),Desayuno),
		listaComida(ListaMediaMananaP, mediaManana, MaxKCalMediaManana-MinKCalMediaManana,  ((MaxKCalMediaManana+MinKCalMediaManana)/2), MediaManana),
		listaComida(ListaComidaP, comida, MaxKCalComida-MinKCalComida,  ((MaxKCalComida+MinKCalComida)/2), Comida),
		listaComida(ListaMeriendaP, merienda, MaxKCalMerienda-MinKCalMerienda,  ((MaxKCalMerienda+MinKCalMerienda)/2), Merienda),
		listaComida(ListaCenaP, cena, MaxKCalCena-MinKCalCena, ((MaxKCalCena+MinKCalCena)/2), Cena),
		
		%comprobamos que no nos pasamos de las calorias, hidratos, proteinas y grasas necesarias ni nos quedamos muy por debajo
		sumaKCals(Desayuno, KCalDesayuno),
		sumaKCals(MediaManana, KCalMediaManana),
		sumaKCals(Comida,KCalComida),
		sumaKCals(Merienda, KCalMerienda),
		sumaKCals(Cena, KCalCena),
		
		sumaHidratos(Desayuno, HidratosDesayuno),
		sumaHidratos(MediaManana, HidratosMediaManana),
		sumaHidratos(Comida,HidratosComida),
		sumaHidratos(Merienda, HidratosMerienda),
		sumaHidratos(Cena, HidratosCena),
		
		sumaProteinas(Desayuno, ProteinasDesayuno),
		sumaProteinas(MediaManana, ProteinasMediaManana),
		sumaProteinas(Comida,ProteinasComida),
		sumaProteinas(Merienda, ProteinasMerienda),
		sumaProteinas(Cena, ProteinasCena),
		
		sumaGrasas(Desayuno, GrasasDesayuno),
		sumaGrasas(MediaManana, GrasasMediaManana),
		sumaGrasas(Comida,GrasasComida),
		sumaGrasas(Merienda, GrasasMerienda),
		sumaGrasas(Cena, GrasasCena),
			
		sum_list([KCalDesayuno, KCalMediaManana, KCalComida, KCalMerienda, KCalCena], KCalTotal),
		sum_list([ProteinasDesayuno,ProteinasMediaManana,ProteinasComida,ProteinasMerienda,ProteinasCena],ProteinasTotal),
		sum_list([HidratosDesayuno,HidratosMediaManana,HidratosComida,HidratosMerienda,HidratosCena],HidratosTotal),
		sum_list([GrasasDesayuno,GrasasMediaManana,GrasasComida,GrasasMerienda,GrasasCena],GrasasTotal),
		
		(Kcal-50)=<KCalTotal, KCalTotal=<(Kcal+50),
		(GrasasMin-50)=<GrasasTotal, GrasasTotal=<(GrasasMax+50),
		(ProteinasMin-50)=<ProteinasTotal, ProteinasTotal=<(ProteinasMax+50), 
		(HidratosMin-50)=<HidratosTotal, HidratosTotal=<(HidratosMax+50).
			

			
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%			
% Devuelve un conjunto de alimentos que pueden ser combinados para una comida. Se considera comer el plato, no comerlo,
% o comer una parte mayor de 50 gramos. Las calorias totales deben estar en el rango min-max calculado
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% listaComida(+L,+TipoCom,+DKCal,+KCal,-L3)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

listaComida(_, _, DKCal,K,[]):- K<(DKCal+1), K>(-1), !. 
listaComida([(NomCom,Peso,KCalAux,P1,H1,G1)|L], TipoCom, DKCal, KCal, [(NomCom,Peso,KCalAux,P1,H1,G1)|L3]):- KCalAux<(KCal+1), listaComida(L, TipoCom, DKCal ,KCal-KCalAux, L3).
listaComida([(NomCom,Peso,KCalAux,P1,H1,G1)|L], TipoCom, DKCal, KCal, [(NomCom,Peso2,K2,P2,H2,G2)|L3]):-  KCalAux>(KCal+1), T is (KCal/KCalAux), Peso2 is floor(Peso*T),  Peso2 > 50, K2 is floor(KCalAux*T), P2 is floor(P1*T), H2 is floor(H1*T), G2 is floor(G1*T), listaComida(L, TipoCom, DKCal, KCal-K2, L3).					
listaComida([(_,_,_,_,_,_)|L], TipoCom, DKCal, KCal,  L3):- listaComida(L, TipoCom, DKCal, KCal, L3).																								
													
		
		
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%					
% Busca comidas que cumplan que las calorias esten entre el 'MinCal' y el 'MaxCal', que esten permitidas en 'Comida', y a las que no se tenga alergia,
% las comidas preferentes se almacenan varias veces en la lista de comidas para aumentar la probabilidad de su aparicion. La lista de alimentos
% se devuelve en 'L'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% buscaComida(+MinCal,+MaxCal,+HoraComida,-L)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%			

buscaComida(MinCal, MaxCal, HoraComida, L):-
	findall((Comida),(es_ingrediente(Comida, X), alergia(X)),LCNP),
	findall((Comida),(es_ingrediente(Comida, X), preferente(X), not(member(Comida, LCNP))), LCP),
	findall((Comida, Gramos, KCalComida, Proteinas, Hidratos, Grasas),(alimento(Comida,Gramos,KCalComida,Proteinas,Hidratos,Grasas), not(member(Comida, LCNP)), esta_permitido(Comida, HoraComida), es_ingrediente(Comida,X), X\='Leche', MinCal=<KCalComida, MaxCal>=KCalComida),Lx), append(LCP, LCP, LCP2), append(LCP2, Lx, L).

	
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Calcula un menu para 7 dias y escribe el resultado en un fichero 'dieta'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calcula_menu_semanal(+KCal, +PorcProteinas, +PorcHidratos, +PorcGrasas)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

calcula_menu_semanal(KCal, PorcProteinas, PorcHidratos, PorcGrasas):- 
							open(menuSemanal, write, Fd), 
							write(Fd, '============================================================================================================\n'),
							write(Fd, '======================================Tu dieta semanal recomendada es=======================================\n'),
							write(Fd, '============================================================================================================\n'),
							calcula_menu_por_dias(KCal, PorcProteinas, PorcHidratos, PorcGrasas, 7, Fd), 
							close(Fd).
							
	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Escribe el menu para un dia, metodo auxiliar usado por calcula_menu_semanal							
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_menu_por_dias(+Kcal, +PorcProteinas, +PorcHidratos, +PorcGrasas, +Dias, +Fd)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

calcula_menu_por_dias(_,_,_,_,0,_). 
calcula_menu_por_dias(Kcal,  PorcProteinas, PorcHidratos, PorcGrasas, Dias, Fd):- 
		calcula_comidas(Kcal,  PorcProteinas, PorcHidratos, PorcGrasas, Desayuno, MediaManana, Comida, Merienda, Cena, KCalTotal2, PTot2, HTot2, GTot2),
		KCalTotal is floor(KCalTotal2), PTot is floor(PTot2), HTot is floor(HTot2), GTot is floor(GTot2),
		Pp is floor(100*(PTot2*4)/KCalTotal2), Ph is floor(100*(HTot2*4)/KCalTotal2), Pg is floor(100*(GTot2*9)/KCalTotal2),
		DiasR is floor(8-Dias),
		write(Fd, '\n\n'),
		write(Fd, '                                                     Día '), write(Fd, DiasR), write(Fd,'                                                     \n'),
		write(Fd, '____________________________________________________________________________________________________________\n\n'),
		write(Fd, 'Dieta de '), write(Fd, KCalTotal), write(Fd, ' calorias: '),  write(Fd, '\n'),
		write(Fd, '------------------------------------------------------------------------------------------------------------\n'),
		write(Fd, '\t'), write(Fd, 'Proteinas:'), write(Fd, '\t'), write(Fd, PTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Pp), write(Fd, '%)\n'),
		write(Fd, '\t'), write(Fd, 'Hidratos: '), write(Fd, '\t'), write(Fd, HTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Ph), write(Fd, '%)\n'),
		write(Fd, '\t'), write(Fd, 'Grasas: '),   write(Fd, '\t'), write(Fd, GTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Pg), write(Fd, '%)\n'),
		write(Fd, '============================================================================================================\n'),
		write(Fd, 'Desayuno: '),  write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, Desayuno), 
		write(Fd, 'Media mañana: '), write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, MediaManana),
		write(Fd, 'Comida: '), write(Fd, '\n'), write(Fd, '\t'),  writeList(Fd, Comida), 
		write(Fd, 'Merienda: '), write(Fd, '\n'), write(Fd, '\t'),  writeList(Fd, Merienda),
		write(Fd, 'Cena: '), write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, Cena),
		Dias1 is Dias - 1, 
		calcula_menu_por_dias(Kcal,  PorcProteinas, PorcHidratos, PorcGrasas, Dias1, Fd),
		!.
calcula_menu_por_dias(Kcal, PorcProteinas, PorcHidratos, PorcGrasas, Dias, Fd) :- 
															calcula_menu_por_dias(Kcal, PorcProteinas, PorcHidratos, PorcGrasas, Dias, Fd).		
		
		
		
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%			
% Calcula las comidas para un dia y escribe el resultado en un fichero 'dieta'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_menu(+Kcal, +PorcProteinas, +PorcHidratos, +PorcGrasas)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

calcula_menu(Kcal, PorcProteinas, PorcHidratos, PorcGrasas) :-
		calcula_comidas(Kcal, PorcProteinas, PorcHidratos, PorcGrasas, Desayuno, MediaManana, Comida, Merienda, Cena, KCalTotal2, PTot2, HTot2, GTot2),
		KCalTotal is floor(KCalTotal2), PTot is floor(PTot2), HTot is floor(HTot2), GTot is floor(GTot2),
		Pp is floor(100*(PTot2*4)/KCalTotal2), Ph is floor(100*(HTot2*4)/KCalTotal2), Pg is floor(100*(GTot2*9)/KCalTotal2),
		open(dieta, write, Fd),
		write(Fd, '============================================================================================================\n'),
		write(Fd, '======================================Tu dieta recomendada para hoy es======================================\n'),
		write(Fd, '============================================================================================================\n'),
		write(Fd, 'Dieta de '), write(Fd, KCalTotal), write(Fd, ' calorias: '),  write(Fd, '\n'),
		write(Fd, '------------------------------------------------------------------------------------------------------------\n'),
		write(Fd, '\t'), write(Fd, 'Proteinas:'), write(Fd, '\t'), write(Fd, PTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Pp), write(Fd, '%)\n'),
		write(Fd, '\t'), write(Fd, 'Hidratos: '), write(Fd, '\t'), write(Fd, HTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Ph), write(Fd, '%)\n'),
		write(Fd, '\t'), write(Fd, 'Grasas: '),   write(Fd, '\t'), write(Fd, GTot), write(Fd, 'g\t'), write(Fd, '('), write(Fd, Pg), write(Fd, '%)\n'),
		write(Fd, '============================================================================================================\n'),
		write(Fd, 'Desayuno: '),  write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, Desayuno), 
		write(Fd, 'Media mañana: '), write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, MediaManana),
		write(Fd, 'Comida: '), write(Fd, '\n'), write(Fd, '\t'),  writeList(Fd, Comida), 
		write(Fd, 'Merienda: '), write(Fd, '\n'), write(Fd, '\t'),  writeList(Fd, Merienda),
		write(Fd, 'Cena: '), write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, Cena),
		close(Fd), !.
		
calcula_menu(Kcal, PorcProteinas, PorcHidratos, PorcGrasas) :- calcula_menu(Kcal,  PorcProteinas, PorcHidratos, PorcGrasas).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Metodo auxiliar para escribir datos en el fichero
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% writeList(-Fd, -L)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

writeList(Fd, []):-  write(Fd, '\n').
writeList(Fd, [(Nom,Peso,Kcal,Prot,Hid,Gras)|ListaComida]):- 
												write(Fd, Nom), write(Fd,':   '),
												write(Fd, Peso), write(Fd,' g, '),
												write(Fd, Kcal), write(Fd,' kcal, '),
												write(Fd, Prot), write(Fd,' g.proteinas, '),
												write(Fd, Hid),  write(Fd,' g.hidratos, '),
												write(Fd, Gras), write(Fd,' g.grasas.'),
												write(Fd, '\n'), write(Fd, '\t'), writeList(Fd, ListaComida) .


												
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%													
% Calcula el indice de masa corporal usando la altua en cm y el peso en kg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_IMC(+Altura, +Peso, -IMC)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

calcula_IMC(Altura, Peso, IMC):- IMC is (Peso/((Altura/100)*(Altura/100))).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%													
% Calculo del ritmo metabolico basal en funcion del sexo, peso(kg), altura(cm) y edad 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_RMB(+Sexo (h o m), +Peso, +Altura, +Edad, -RMB)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

calcula_RMB(h, Peso, Altura, Edad, RMB):-
	RMB is 66.473 + ((13.752*Peso) + ((5.0033*Altura)-(6.755*Edad))),!.
calcula_RMB(m, Peso, Altura, Edad, RMB):-
	RMB is 655.0955 + ((9.463*Peso) + ((1.8496*Altura)-(4.6756*Edad))).

	
	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Calculo de las calorias necesarias en funcion de: sexo, peso(kg), altura(cm), edad	y actividad fisica
% Actividad fisica puede ser: sedentaria, ligera, media, activa, muyActiva
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_Kcal(+Sexo, +Peso, +Altura, +Edad, +ActividadFisica, -Kcal)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

calcula_Kcal(Sexo, Peso, Altura, Edad, sedentaria, Kcal):-
	calcula_RMB(Sexo, Peso, Altura, Edad, RMB),
	Kcal is RMB*1.2,!.
calcula_Kcal(Sexo, Peso, Altura, Edad, ligera, Kcal):-
	calcula_RMB(Sexo, Peso, Altura, Edad, RMB),
	Kcal is RMB*1.375,!.
calcula_Kcal(Sexo, Peso, Altura, Edad, media, Kcal):-
	calcula_RMB(Sexo, Peso, Altura, Edad, RMB),
	Kcal is RMB*1.55,!.
calcula_Kcal(Sexo, Peso, Altura, Edad, activa, Kcal):-
	calcula_RMB(Sexo, Peso, Altura, Edad, RMB),
	Kcal is RMB*1.725,!.
calcula_Kcal(Sexo, Peso, Altura, Edad, muyActiva, Kcal):-
	calcula_RMB(Sexo, Peso, Altura, Edad, RMB),
	Kcal is RMB*1.9, !.


	
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% consulta para ver si el alimento se puede comer a esa hora
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% esta_permitido(-ComidaAComer, -MomentoDeComer)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

esta_permitido(ComidaAComer, MomentoDeComer):- permitido(ComidaAComer, Momentos), member(MomentoDeComer,Momentos).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Calcula el porcentaje maximo y minimo de proteinas, hidratos y grasas respecto a un numero de calorias determinado
% y a un porcentaje base pasado por parametro de entrada
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% calcula_cantidades(+KCal, -GrasasMin, -ProteinasMin, -HidratosMin, -GrasasMax, -ProteinasMax, -HidratosMax)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
	
calcula_cantidades(KCal, PorcProteinas, PorcHidratos, PorcGrasas, GrasasMin, ProteinasMin, HidratosMin, GrasasMax, ProteinasMax, HidratosMax):-
													GrasasMin is (((PorcGrasas-5)*KCal)/9),   
													ProteinasMin is (((PorcProteinas-5)*KCal)/4),
													HidratosMin is (((PorcHidratos-5)*KCal)/4),
													GrasasMax is (((PorcGrasas+5)*KCal)/9),   
													ProteinasMax is (((PorcProteinas+5)*KCal)/4),
													HidratosMax is (((PorcHidratos+5)*KCal)/4).

													
													
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% Metodos auxiliares para calcular el valor total de calorias/hidratos/proteinas/grasas del menu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
% sumaKCals(+L, -Res).
% sumaHidratos(+L, -Res).
% sumaProteinas(+L, -Res).
% sumaGrasas(+L, -Res).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%		

sumaKCals([], 0).
sumaKCals([(_,_,KCalAux,_,_,_)|ListaComida], Res):- sumaKCals(ListaComida, Res1), Res is Res1 + KCalAux.

sumaHidratos([], 0).
sumaHidratos([(_,_,_,_,H,_)|ListaComida], Res):- sumaHidratos(ListaComida, Res1), Res is Res1 + H.

sumaProteinas([], 0).
sumaProteinas([(_,_,_,P,_,_)|ListaComida], Res):- sumaProteinas(ListaComida, Res1), Res is Res1 + P.

sumaGrasas([], 0).
sumaGrasas([(_,_,_,_,_,G)|ListaComida], Res):- sumaGrasas(ListaComida, Res1), Res is Res1 + G.
